from django.contrib.auth.models import User
from rest_framework import status

from ichec_django_core.models import Organization, Address

from ichec_django_core.utils.test_utils.test_client import (
    setup_default_users_and_groups,
    add_group_permissions,
    AuthAPITestCase,
)


class OgranizationViewTests(AuthAPITestCase):

    template = {
        "name": "My Org",
        "acronym": "MO",
        "description": "An description of the org",
        "address": {"line1": "1234 Street", "region": "Region", "country": "IE"},
        "website": "www.org.org",
    }
    update_template = {"address": {"country": "FR"}}

    def setUp(self):
        self.url = "/api/organizations/"

        setup_default_users_and_groups()

        add_group_permissions(
            "admins",
            Organization,
            ["change_organization", "add_organization"],
        )

        add_group_permissions(
            "admins",
            Address,
            ["change_address", "add_address"],
        )

    def test_list_not_authenticated(self):
        self.assert_401(self.do_list())

    def test_detail_not_authenticated(self):
        self.assert_401(self.detail(1))

    def test_list_authenticated(self):
        self.assert_200(self.authenticated_list("regular_user"))

    def test_create_not_authenticated(self):
        self.assert_401(self.create(self.template))

    def test_create_authenticated_no_permission(self):
        self.assert_403(self.authenticated_create("regular_user", self.template))

    def test_create_authenticated_permission(self):
        self.assert_201(self.authenticated_create("admin_user", self.template))

    def test_update_authenticated_permission(self):
        created = self.assert_201(
            self.authenticated_create("admin_user", self.template)
        )

        for key, value in self.update_template.items():
            if isinstance(value, dict):
                for sub_key, sub_value in value.items():
                    created[key][sub_key] = sub_value

        updated = self.assert_200(self.authenticated_update("admin_user", 1, created))

        for key, value in self.update_template.items():
            if isinstance(value, dict):
                for sub_key, sub_value in value.items():
                    self.assertEqual(updated[key][sub_key], sub_value)



from django.db import models

from .member import Member
from .utils import TimesStampMixin


class Feedback(TimesStampMixin):

    creator = models.ForeignKey(Member, on_delete=models.CASCADE)
    comments = models.TextField()



from rest_framework import serializers

from ichec_django_core.models import Member

from .core import SERIALIZER_BASE_FIELDS


class MemberBaseSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        model = Member
        fields = SERIALIZER_BASE_FIELDS + (
            "username",
            "email",
            "first_name",
            "last_name",
            "phone",
            "profile",
        )
        read_only_fields = SERIALIZER_BASE_FIELDS


class MemberResponseSerializer(MemberBaseSerializer):

    profile = serializers.HyperlinkedIdentityField(
        read_only=True, view_name="member_profiles"
    )
    profile_thumbnail = serializers.HyperlinkedIdentityField(
        read_only=True, view_name="member_profile_thumbnails"
    )

    class Meta:
        model = Member
        fields = MemberBaseSerializer.Meta.fields + (
            "organizations",
            "profile_thumbnail",
        )
        read_only_fields = ("profile", "profile_thumbnail", "organizations")

    def to_representation(self, instance):
        rep = super().to_representation(instance)
        if not instance.profile:
            rep["profile"] = None
        if not instance.profile_thumbnail:
            rep["profile_thumbnail"] = None
        return rep


class MemberDetailResponseSerializer(MemberResponseSerializer):

    permissions = serializers.SerializerMethodField("get_all_permissions")

    class Meta:
        model = Member
        fields = MemberResponseSerializer.Meta.fields + ("permissions",)
        read_only_fields = MemberResponseSerializer.Meta.read_only_fields + (
            "permissions",
        )

    def get_all_permissions(self, obj):
        return obj.get_all_permissions()


class MemberListSerializer(MemberBaseSerializer):
    class Meta:
        model = Member
        fields = MemberBaseSerializer.Meta.fields
        read_only_fields = MemberBaseSerializer.Meta.read_only_fields

    def to_representation(self, instance):
        return MemberResponseSerializer(context=self.context).to_representation(
            instance
        )


class MemberDetailSerializer(MemberBaseSerializer):
    class Meta:
        model = Member
        fields = MemberBaseSerializer.Meta.fields
        read_only_fields = MemberBaseSerializer.Meta.read_only_fields

    def to_representation(self, instance):
        return MemberDetailResponseSerializer(context=self.context).to_representation(
            instance
        )



from django.db import models

from django_countries.fields import CountryField

from .utils import TimesStampMixin

from .member import Member


class Address(TimesStampMixin):

    line1 = models.CharField(max_length=200)
    line2 = models.CharField(max_length=200, null=True)
    line3 = models.CharField(max_length=200, null=True)
    city = models.CharField(max_length=200, null=True)
    region = models.CharField(max_length=200)
    postcode = models.CharField(max_length=200, null=True)
    country = CountryField()

    class Meta:
        verbose_name_plural = "Addresses"

    def __str__(self):
        return f"{self.line1}, {self.region}, {self.country}"

    @property
    def country_name(self):
        return self.country.name

    @property
    def country_flag(self):
        return self.country.flag


class Organization(TimesStampMixin):

    name = models.CharField(max_length=200)
    acronym = models.CharField(max_length=100, blank=True, null=True)
    description = models.TextField(blank=True)
    address = models.ForeignKey(Address, on_delete=models.CASCADE)
    website = models.CharField(max_length=250, blank=True, null=True)
    members = models.ManyToManyField(Member, blank=True, related_name="organizations")

    def __str__(self):
        return self.name

    @property
    def is_facility(self):
        return hasattr(self, "facility")



from rest_framework import status
from rest_framework.test import APITestCase


from ichec_django_core.utils.test_utils.test_client import (
    setup_default_users_and_groups,
)


class TopLevelViewTests(APITestCase):
    def setUp(self):

        setup_default_users_and_groups()

    def test_get_org_not_authenticated(self):
        response = self.client.get("/api/", format="json")
        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)



from django.apps import AppConfig


class IchecDjangoCoreConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "ichec_django_core"
    verbose_name = "ICHEC Django Core"



from pathlib import Path
import shutil
import os
import uuid

from PIL import Image

from django.db import models
from django.utils import timezone
from django.conf import settings


class TimesStampMixin(models.Model):
    created_at = models.DateTimeField(default=timezone.now, editable=False)
    updated_at = models.DateTimeField(default=timezone.now)

    class Meta:
        abstract = True


def make_zip(work_path: Path, output_path: Path):
    shutil.make_archive(str(output_path), "zip", str(work_path))


def generate_thumbnail(
    model: models.Model, field_name: str, field: models.ImageField, size=(128, 128)
) -> str:

    media_root = Path(settings.MEDIA_ROOT)
    storage_dir = Path(f"{model._meta.model_name}/{field_name}/thumbnails")
    print(media_root / storage_dir)
    os.makedirs(media_root / storage_dir, exist_ok=True)

    image = Image.open(field)
    image.thumbnail(size)
    file_path = storage_dir / f"{uuid.uuid4()}.png"
    image.save(media_root / file_path, "PNG")
    print(file_path)
    return str(file_path)


def content_file_name(field: str, instance, filename: str):
    """
    Handler for the Django File upload_to field for user media.
    Store the media in a prefixed directory by model and
    make sure there is a unique filename
    """
    instance.original_file_name = filename
    _, ext = os.path.splitext(filename)

    storage_dir = Path(f"{instance._meta.model_name}/{field}")
    return storage_dir / f"{uuid.uuid4()}{ext}"



"""
Django settings for marinerg_tap project.

Generated by 'django-admin startproject' using Django 5.1.

For more information on this file, see
https://docs.djangoproject.com/en/5.1/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/5.1/ref/settings/
"""

from pathlib import Path
import os

# Build paths inside the project like this: BASE_DIR / 'subdir'.
BASE_DIR = Path(__file__).resolve().parent.parent

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/5.1/howto/deployment/checklist/

secret_key_env = os.environ.get("DJANGO_SECRET_KEY")
if not secret_key_env:
    raise RuntimeError("No DJANGO_SECRET_KEY key set in environment")
SECRET_KEY = secret_key_env.replace('"', "")

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = bool(os.environ.get("DJANGO_DEBUG", default=0))

ALLOWED_HOSTS = os.environ.get("DJANGO_ALLOWED_HOSTS", default="*").split(",")

ADMIN_URL = os.environ.get("DJANGO_ADMIN_URL", default="admin")

API_AUTH_URL = os.environ.get("DJANGO_API_AUTH_URL", default="api-auth")

# OIDC Connection
WITH_OIDC = bool(os.environ.get("WITH_OIDC", default=0))
OIDC_RP_CLIENT_ID = os.environ.get("OIDC_RP_CLIENT_ID")
OIDC_RP_CLIENT_SECRET = os.environ.get("OIDC_RP_CLIENT_SECRET")
OIDC_RP_SIGN_ALGO = "RS256"
OIDC_OP_AUTHORIZATION_ENDPOINT = os.environ.get("OIDC_OP_AUTHORIZATION_ENDPOINT")
OIDC_OP_TOKEN_ENDPOINT = os.environ.get("OIDC_OP_TOKEN_ENDPOINT")
OIDC_OP_USER_ENDPOINT = os.environ.get("OIDC_OP_USER_ENDPOINT")
OIDC_OP_JWKS_ENDPOINT = os.environ.get("OIDC_OP_JWKS_ENDPOINT")

# Application definition
INSTALLED_APPS = [
    "ichec_django_core.apps.IchecDjangoCoreConfig",
    "rest_framework",
    "corsheaders",
    "django_countries",
    "rest_framework.authtoken",
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
]

if WITH_OIDC:
    INSTALLED_APPS.append("mozilla_django_oidc")
    LOGIN_REDIRECT_URL = "/"
    LOGOUT_REDIRECT_URL = "/"

MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "corsheaders.middleware.CorsMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
]

if WITH_OIDC:
    MIDDLEWARE.append("mozilla_django_oidc.middleware.SessionRefresh")


# Auth
AUTHENTICATION_BACKENDS = ["django.contrib.auth.backends.ModelBackend"]
if WITH_OIDC:
    AUTHENTICATION_BACKENDS = [
        "mozilla_django_oidc.auth.OIDCAuthenticationBackend"
    ] + AUTHENTICATION_BACKENDS

ROOT_URLCONF = "app.urls"


def get_templates(base_dir: Path):
    return [
        {
            "BACKEND": "django.template.backends.django.DjangoTemplates",
            "DIRS": [os.path.join(base_dir, "templates")],
            "APP_DIRS": True,
            "OPTIONS": {
                "context_processors": [
                    "django.template.context_processors.debug",
                    "django.template.context_processors.request",
                    "django.contrib.auth.context_processors.auth",
                    "django.contrib.messages.context_processors.messages",
                ],
            },
        },
    ]


TEMPLATES = get_templates(BASE_DIR)

WSGI_APPLICATION = "app.wsgi.application"


# Database
# https://docs.djangoproject.com/en/5.1/ref/settings/#databases


def get_databases(base_dir: Path):
    return {
        "default": {
            "ENGINE": os.environ.get("DJANGO_SQL_ENGINE", "django.db.backends.sqlite3"),
            "NAME": os.environ.get("DJANGO_SQL_DATABASE", base_dir / "db.sqlite3"),
            "USER": os.environ.get("DJANGO_SQL_USER", "user"),
            "PASSWORD": os.environ.get("DJANGO_SQL_PASSWORD", "password"),
            "HOST": os.environ.get("DJANGO_SQL_HOST", "localhost"),
            "PORT": os.environ.get("DJANGO_SQL_PORT", "5432"),
        }
    }


DATABASES = get_databases(BASE_DIR)


# Password validation
# https://docs.djangoproject.com/en/5.1/ref/settings/#auth-password-validators

pv_prefix = "django.contrib.auth.password_validation"
AUTH_PASSWORD_VALIDATORS = [
    {
        "NAME": f"{pv_prefix}.UserAttributeSimilarityValidator",
    },
    {
        "NAME": f"{pv_prefix}.MinimumLengthValidator",
    },
    {
        "NAME": f"{pv_prefix}.CommonPasswordValidator",
    },
    {
        "NAME": f"{pv_prefix}.NumericPasswordValidator",
    },
]


# Internationalization
# https://docs.djangoproject.com/en/5.1/topics/i18n/

LANGUAGE_CODE = "en-us"

TIME_ZONE = "UTC"

USE_I18N = True

USE_TZ = True

SECURE_PROXY_SSL_HEADER = ("HTTP_X_FORWARDED_PROTO", "https")

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/5.1/howto/static-files/

STATIC_URL = "static/"


def get_static_root(base_dir: Path):
    return Path(os.environ.get("DJANGO_STATIC_ROOT", default=base_dir / "staticfiles"))


STATIC_ROOT = get_static_root(BASE_DIR)


def get_media_root(base_dir: Path):
    return Path(os.environ.get("DJANGO_MEDIA_ROOT", default=base_dir / "mediafiles"))


MEDIA_ROOT = get_media_root(BASE_DIR)

DEFAULT_FILE_STORAGE = "django_downloadview.storage.SignedStorage"
DOWNLOADVIEW_BACKEND = "django_downloadview.nginx.XAccelRedirectMiddleware"

DOWNLOADVIEW_RULES = [
    {
        "source_url": "/media/nginx/",
        "destination_url": "/nginx-optimized-by-middleware/",
    },
]

# Default primary key field type
# https://docs.djangoproject.com/en/5.1/ref/settings/#default-auto-field

DEFAULT_AUTO_FIELD = "django.db.models.BigAutoField"

if WITH_OIDC:
    DRF_DEFAULT_AUTH = "mozilla_django_oidc.contrib.drf.OIDCAuthentication"
else:
    DRF_DEFAULT_AUTH = "rest_framework.authentication.TokenAuthentication"

REST_FRAMEWORK = {
    "DEFAULT_PERMISSION_CLASSES": [
        "rest_framework.permissions.IsAuthenticated",
    ],
    "DEFAULT_AUTHENTICATION_CLASSES": [
        DRF_DEFAULT_AUTH,
        "rest_framework.authentication.SessionAuthentication",
    ],
    "DEFAULT_PAGINATION_CLASS": "ichec_django_core.view_sets.pagination.PageNumberWithSizePagination",  # NOQA
    "PAGE_SIZE": 100,
}

CORS_ALLOWED_ORIGINS = [
    "http://localhost:4200",
]

if False:
    EMAIL_HOST_USER = "Test User"
    EMAIL_BACKEND = "django.core.mail.backends.filebased.EmailBackend"
    EMAIL_FILE_PATH = os.getcwd() + "/app-messages"
else:
    EMAIL_HOST = os.environ.get("DJANGO_EMAIL_HOST", default="")
    EMAIL_PORT = os.environ.get("DJANGO_EMAIL_PORT", default="")
    EMAIL_HOST_USER = os.environ.get("DJANGO_EMAIL_HOST_USER", default="")
    EMAIL_HOST_PASSWORD = os.environ.get("DJANGO_EMAIL_HOST_PASSWORD", default="")
    EMAIL_USE_TLS = True

LOGGING = {
    "version": 1,
    "disable_existing_loggers": False,
    "handlers": {
        "console": {
            "class": "logging.StreamHandler",
        },
    },
    "root": {
        "handlers": ["console"],
        "level": "DEBUG",
    },
}



from .member import MemberListSerializer, MemberDetailSerializer

from .core import (
    GroupSerializer,
    NestedHyperlinkedModelSerializer,
    SERIALIZER_BASE_FIELDS,
)

from .organization import (
    OrganizationListSerializer,
    OrganizationDetailSerializer,
    AddressSerializer,
)

from .feedback import FeedbackSerializer

__all__ = [
    "SERIALIZER_BASE_FIELDS",
    "NestedHyperlinkedModelSerializer",
    "MemberListSerializer",
    "MemberDetailSerializer",
    "FeedbackSerializer",
    "GroupSerializer",
    "AddressSerializer",
    "OrganizationListSerializer",
    "OrganizationDetailSerializer",
    "PermissionSerializer",
]



from rest_framework import serializers

from django_countries.serializers import CountryFieldMixin

from ichec_django_core.models import Organization, Address

from .core import SERIALIZER_BASE_FIELDS, NestedHyperlinkedModelSerializer


class AddressSerializer(CountryFieldMixin, serializers.HyperlinkedModelSerializer):
    class Meta:
        model = Address
        fields = SERIALIZER_BASE_FIELDS + (
            "line1",
            "line2",
            "line3",
            "city",
            "region",
            "postcode",
            "country",
            "country_name",
            "country_flag",
        )
        read_only_fields = SERIALIZER_BASE_FIELDS + ("country_name", "country_flag")


class OrganizationBaseSerializer(NestedHyperlinkedModelSerializer):
    class Meta:
        model = Organization
        fields = NestedHyperlinkedModelSerializer.base_fields + (
            "name",
            "acronym",
            "description",
            "address",
            "website",
            "members",
        )
        read_only_fields = NestedHyperlinkedModelSerializer.base_fields


class OrganizationListSerializer(OrganizationBaseSerializer):
    pass


class OrganizationDetailSerializer(OrganizationBaseSerializer):
    address = AddressSerializer()

    def create(self, validated_data):
        address_data = validated_data.pop("address")
        address = Address.objects.create(**address_data)

        many_to_many = self.pop_many_to_many(validated_data)
        instance = Organization.objects.create(address=address, **validated_data)
        self.add_many_to_many(instance, many_to_many)
        return instance

    def update(self, instance, validated_data):
        address_data = validated_data.pop("address")

        instance = super().update(instance, validated_data)
        for attr, value in address_data.items():
            setattr(instance.address, attr, value)
        return instance



import logging

import requests

logger = logging.getLogger(__name__)


class RestClient:
    def __init__(self, endpoint: str, username: str, password: str):
        self.endpoint = endpoint
        self.token = self._get_token(username, password)

    @property
    def default_headers(self) -> dict:
        return {"Authorization": f"Token {self.token}"}

    def _get_token(self, username: str, password: str) -> str:
        r = requests.post(
            f"{self.endpoint}/api-token-auth/",
            data={"username": username, "password": password},
        )

        assert r.status_code == 200
        return r.json()["token"]

    def post(self, url: str, data: dict, files=None) -> dict:
        if files:
            r = requests.post(
                f"{self.endpoint}/api/{url}/",
                headers=self.default_headers,
                data=data,
                files={"image": files},
            )
        else:
            r = requests.post(
                f"{self.endpoint}/api/{url}/", headers=self.default_headers, json=data
            )
        if r.status_code >= 400:
            logger.error("Post failed with: %s\n%s", r.status_code, r.text)
            r.raise_for_status()
        return r.json()

    def patch(self, url: str, data: dict) -> dict:
        r = requests.patch(
            f"{self.endpoint}/api/{url}/", headers=self.default_headers, json=data
        )
        if r.status_code >= 400:
            logger.error("Patch failed with: %s\n%s", r.status_code, r.json())
            r.raise_for_status()
        return r.json()

    def get(self, url: str, query: str = "", fail_if_empty: bool = True) -> None | dict:
        r = requests.get(
            f"{self.endpoint}/api/{url}/{query}", headers=self.default_headers
        )
        if r.status_code == 404:
            if fail_if_empty:
                r.raise_for_status()
            return None
        if r.status_code >= 400:
            logger.error("Get failed with: %s\n%s", r.status_code, r.json())
            r.raise_for_status()
        return r.json()

    def delete(self, url: str):
        r = requests.delete(f"{self.endpoint}/api/{url}", headers=self.default_headers)
        if r.status_code >= 400:
            logger.error("Delete failed with: %s", r.status_code)
            r.raise_for_status()



from .core import *  # NOQA
from .feedback import *  # NOQA
from .organization import *  # NOQA
from .member import *  # NOQA
from .permissions import *  # NOQA
from .files import ObjectDownloadView, ObjectFileDownloadView, ObjectFileUploadView

__all__ = ["ObjectDownloadView", "ObjectFileDownloadView", "ObjectFileUploadView"]



import logging

from pydantic import BaseModel

from .rest_client import RestClient
from .models import (
    get_path_from_item,
    get_path_from_type,
    get_type,
    Resource,
    Timestamped,
)

logger = logging.getLogger(__name__)


class Paginated(BaseModel, frozen=True):

    count: int = 0
    next: str | None = None
    previous: str | None = None
    results: list[Timestamped] = []


class PortalClient(RestClient):

    def get_items(self, item_t, page: int = 1) -> Paginated:
        path = get_path_from_type(item_t)
        return_t = get_type(path, "list")

        ret_json = self.get(path)
        if ret_json:
            return Paginated(
                count=ret_json["count"],
                next=ret_json["next"],
                previous=ret_json["previous"],
                results=[return_t(**item) for item in ret_json["results"]],
            )
        else:
            return Paginated()

    def create_item(self, resource: BaseModel) -> Timestamped:
        path = get_path_from_item(resource)

        create_json = self.post(path, resource.model_dump(mode="json"))
        return get_type(path, "detail")(**create_json)

    def create_items(self, resources: list[BaseModel]) -> list[Timestamped]:
        return [self.create_item(item) for item in resources]

    def delete_item(self, resource: Resource):
        self.delete(f"{get_path_from_item(resource)}/{resource.id}/")



from django.contrib.auth.models import Group

from rest_framework import serializers
from rest_framework.utils import model_meta


SERIALIZER_BASE_FIELDS = ("id", "url", "created_at", "updated_at")


class NestedHyperlinkedModelSerializer(serializers.HyperlinkedModelSerializer):

    base_fields = SERIALIZER_BASE_FIELDS

    def pop_many_to_many(self, validated_data) -> dict:
        ModelClass = self.Meta.model
        info = model_meta.get_field_info(ModelClass)
        many_to_many = {}
        for field_name, relation_info in info.relations.items():
            if relation_info.to_many and (field_name in validated_data):
                many_to_many[field_name] = validated_data.pop(field_name)
        return many_to_many

    def add_many_to_many(self, instance, many_to_many):
        if not many_to_many:
            return

        for field_name, value in many_to_many.items():
            field = getattr(instance, field_name)
            field.set(value)


class GroupSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = Group
        fields = ["url", "name", "id"]



# Generated by Django 5.2.1 on 2025-10-29 16:48

import django.contrib.auth.models
import django.db.models.deletion
import django.utils.timezone
import django_countries.fields
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("auth", "0012_alter_user_first_name_max_length"),
    ]

    operations = [
        migrations.CreateModel(
            name="Address",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        default=django.utils.timezone.now, editable=False
                    ),
                ),
                ("updated_at", models.DateTimeField(default=django.utils.timezone.now)),
                ("line1", models.CharField(max_length=200)),
                ("line2", models.CharField(max_length=200, null=True)),
                ("line3", models.CharField(max_length=200, null=True)),
                ("city", models.CharField(max_length=200, null=True)),
                ("region", models.CharField(max_length=200)),
                ("postcode", models.CharField(max_length=200, null=True)),
                ("country", django_countries.fields.CountryField(max_length=2)),
            ],
            options={
                "verbose_name_plural": "Addresses",
            },
        ),
        migrations.CreateModel(
            name="Member",
            fields=[
                (
                    "user_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        default=django.utils.timezone.now, editable=False
                    ),
                ),
                ("updated_at", models.DateTimeField(default=django.utils.timezone.now)),
                ("phone", models.CharField(blank=True, max_length=100, null=True)),
                ("profile", models.ImageField(null=True, upload_to="")),
                ("profile_thumbnail", models.ImageField(null=True, upload_to="")),
            ],
            options={
                "verbose_name": "Portal Member",
                "verbose_name_plural": "Portal Members",
            },
            bases=("auth.user", models.Model),
            managers=[
                ("objects", django.contrib.auth.models.UserManager()),
            ],
        ),
        migrations.CreateModel(
            name="Feedback",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        default=django.utils.timezone.now, editable=False
                    ),
                ),
                ("updated_at", models.DateTimeField(default=django.utils.timezone.now)),
                ("comments", models.TextField()),
                (
                    "creator",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="ichec_django_core.member",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="Organization",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        default=django.utils.timezone.now, editable=False
                    ),
                ),
                ("updated_at", models.DateTimeField(default=django.utils.timezone.now)),
                ("name", models.CharField(max_length=200)),
                ("acronym", models.CharField(blank=True, max_length=100, null=True)),
                ("description", models.TextField(blank=True)),
                ("website", models.CharField(blank=True, max_length=250, null=True)),
                (
                    "address",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="ichec_django_core.address",
                    ),
                ),
                (
                    "members",
                    models.ManyToManyField(
                        blank=True,
                        related_name="organizations",
                        to="ichec_django_core.member",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
    ]



from django.contrib.auth.models import User
from rest_framework import status

from ichec_django_core.models import Feedback

from ichec_django_core.utils.test_utils.test_client import (
    setup_default_users_and_groups,
    add_group_permissions,
    AuthAPITestCase,
)


class FeedbackViewTests(AuthAPITestCase):

    template = {"comments": "This is some feedback"}
    update_template = {"comments": "This is some edited feedback"}

    def setUp(self):
        self.url = "/api/feedback/"

        setup_default_users_and_groups()

        add_group_permissions(
            "admins",
            Feedback,
            ["add_feedback"],
        )

        add_group_permissions(
            "regular_users",
            Feedback,
            ["add_feedback"],
        )

    def test_list_not_authenticated(self):
        self.assert_401(self.do_list())

    def test_detail_not_authenticated(self):
        self.assert_401(self.detail(1))

    def test_list_authenticated(self):
        self.assert_200(self.authenticated_list("regular_user"))

    def test_create_not_authenticated(self):
        self.assert_401(self.create(self.template))

    def test_create_regular_user(self):
        self.assert_201(self.authenticated_create("regular_user", self.template))

    def test_create_admin_user(self):
        self.assert_201(self.authenticated_create("admin_user", self.template))

    def test_update_admin_user(self):
        created = self.assert_201(
            self.authenticated_create("admin_user", self.template)
        )

        for key, value in self.update_template.items():
            created[key] = value

        updated = self.assert_200(self.authenticated_update("admin_user", 1, created))

        for key, value in self.update_template.items():
            self.assertEqual(updated[key], value)



from ichec_django_core.utils.test_utils.test_client import (
    AuthAPITestCase,
    setup_default_users_and_groups,
)


class MemberViewTests(AuthAPITestCase):
    def setUp(self):
        self.url = "/api/members/"
        setup_default_users_and_groups()

    def test_list_not_authenticated(self):
        self.assert_401(self.do_list())

    def test_detail_not_authenticated(self):
        self.assert_401(self.detail(1))

    def test_list_authenticated(self):
        self.assert_200(self.authenticated_list("regular_user"))

    def test_detail_authenticated(self):
        self.assert_200(self.authenticated_detail("regular_user", 1))

    def test_create_not_authenticated(self):
        data = {"username": "test_user"}
        self.assert_401(self.create(data))

    def test_create_authenticated_no_permission(self):
        data = {"username": "test_user"}
        self.assert_403(self.authenticated_create("regular_user", data))



from rest_framework import permissions
from rest_framework.authtoken.views import ObtainAuthToken
from rest_framework.authtoken.models import Token
from rest_framework.response import Response

from ichec_django_core.models import Member

from ichec_django_core.serializers import MemberListSerializer, MemberDetailSerializer

from .core import SearchableModelViewSet
from .files import ObjectFileDownloadView, ObjectFileUploadView
from .permissions import SelfEditOrDjangoModelPermissions


class MemberViewSet(SearchableModelViewSet):

    queryset = Member.objects.all()
    serializer_class = MemberListSerializer
    permission_classes = [
        permissions.IsAuthenticated,
        SelfEditOrDjangoModelPermissions,
    ]
    ordering_fields = SearchableModelViewSet.ordering_fields + ("username",)
    ordering: tuple[str, ...] = ("username",)
    search_fields = ["username", "first_name", "last_name", "email"]

    serializers = {
        "retrieve": MemberDetailSerializer,
        "list": MemberListSerializer,
        "create": MemberDetailSerializer,
        "update": MemberDetailSerializer,
        "partial_update": MemberDetailSerializer,
    }

    def get_queryset(self):
        queryset = Member.objects.all().order_by("id")
        org_id = self.request.query_params.get("organization")
        if org_id is not None:
            queryset = queryset.filter(organizations__id=org_id)
        call_id = self.request.query_params.get("call")
        if call_id is not None:
            queryset = queryset.filter(access_boards__id=call_id)
        return queryset


class MemberProfileDownloadView(ObjectFileDownloadView):
    model = Member
    file_field = "profile"
    permissions_class = permissions.DjangoModelPermissions


class MemberProfileThumbnailDownloadView(ObjectFileDownloadView):
    model = Member
    file_field = "profile_thumbnail"
    permissions_class = permissions.DjangoModelPermissions


class MemberProfileUploadView(ObjectFileUploadView):
    model = Member
    queryset = Member.objects.all()
    file_field = "profile"
    permission_classes = [
        permissions.IsAuthenticated,
        SelfEditOrDjangoModelPermissions,
    ]


class CustomAuthToken(ObtainAuthToken):

    authentication_classes: list = []

    """
    This overrides DRFs built in api auth token view
    so we return some more user details. This helps clients
    fetch the user post auth.
    """

    def post(self, request, *args, **kwargs):
        serializer = self.serializer_class(
            data=request.data, context={"request": request}
        )
        serializer.is_valid(raise_exception=True)
        user = serializer.validated_data["user"]
        token, created = Token.objects.get_or_create(user=user)

        return Response({"token": token.key, "user_id": user.pk})



import importlib
import uuid
import os
from pathlib import Path

from django.core.exceptions import PermissionDenied
from django.conf import settings
from django.core.files.storage import FileSystemStorage

from rest_framework.request import Request
from rest_framework.permissions import (
    DjangoObjectPermissions,
    IsAuthenticated,
    DjangoModelPermissions,
)
from rest_framework.parsers import FileUploadParser
from rest_framework.response import Response
from rest_framework.generics import GenericAPIView

from django_downloadview import ObjectDownloadView

rest_auth = importlib.import_module("rest_framework.authentication")

_AUTH_CLASSES = []
for c in settings.REST_FRAMEWORK["DEFAULT_AUTHENTICATION_CLASSES"]:
    modname = ".".join(c.split(".")[:-1])
    mod = importlib.import_module(modname)
    _AUTH_CLASSES.append(getattr(mod, c.split(".")[-1]))


class ObjectFileUploadView(GenericAPIView):

    parser_classes = [FileUploadParser]
    permission_classes = [
        IsAuthenticated,
        DjangoModelPermissions,
    ]

    def put(self, request, *args, **kwargs):
        media_root = Path(settings.MEDIA_ROOT)
        storage_dir = Path(f"{self.model._meta.model_name}/{self.file_field}")
        os.makedirs(media_root / storage_dir, exist_ok=True)

        filename = request.data["file"].name

        _, ext = os.path.splitext(filename)

        file_path = f"{uuid.uuid4()}{ext}"
        written_path = FileSystemStorage(location=media_root / storage_dir).save(
            file_path, request.data["file"]
        )

        instance = self.get_object()
        getattr(instance, self.file_field).original_file_name = filename
        getattr(instance, self.file_field).name = str(storage_dir / written_path)
        instance.save()

        return Response(status=204)


class ObjectFileDownloadView(ObjectDownloadView):

    permissions_class = DjangoObjectPermissions

    def authenticate(self, request) -> None:
        for auth_class in _AUTH_CLASSES:
            auth_resp = auth_class().authenticate(Request(request))
            if auth_resp is not None:
                request.user = auth_resp[0]
                return

    def has_permission(self, request) -> None:

        instance = self.get_object()
        permissions = self.permissions_class()
        if not (
            permissions.has_permission(request, self)
            and permissions.has_object_permission(request, self, instance)
        ):
            raise PermissionDenied()

    def get_basename(self):
        filename = getattr(self.object, self.file_field).name
        return str(self.file_field + "." + filename.split(".")[-1])

    def get(self, request, *args, **kwargs):
        self.authenticate(request)
        self.has_permission(request)
        return super().get(request, *args, **kwargs)



from rest_framework import permissions

from ichec_django_core.models import Feedback, Member

from ichec_django_core.serializers import FeedbackSerializer

from .permissions import OwnerFullOrDjangoModelPermissions
from .core import BaseModelViewSet


class CreatorFullOrDjangoModelPermissions(OwnerFullOrDjangoModelPermissions):
    owner_field = "creator"


class FeedbackViewSet(BaseModelViewSet):

    queryset = Feedback.objects.all()
    serializer_class = FeedbackSerializer
    permission_classes = [
        permissions.IsAuthenticated,
        CreatorFullOrDjangoModelPermissions,
    ]

    def filter_self(self, queryset):
        return queryset.filter(creator__id=self.request.user.id)

    def get_queryset(self):
        """
        If the user has view permissions for this model then return all objects,
        otherwise filter by their own objects only.
        """

        queryset = Feedback.objects.all().order_by("id")

        queryset = self.filter_self_or_permission(queryset, Feedback)
        if not queryset:
            return queryset

        creator_id = self.request.query_params.get("creator")
        if creator_id is not None:
            queryset = queryset.filter(creator__id=creator_id)
        return queryset

    def perform_create(self, serializer):
        serializer.save(creator=Member.objects.get(id=self.request.user.id))



from rest_framework import permissions, viewsets

from ichec_django_core.models import Organization, Address

from ichec_django_core.serializers import (
    OrganizationDetailSerializer,
    OrganizationListSerializer,
    AddressSerializer,
)

from .core import SearchableModelViewSet
from .permissions import MemberEditOrDjangoModelPermissions


class OrganizationViewSet(SearchableModelViewSet):
    queryset = Organization.objects.all()
    serializer_class = OrganizationListSerializer
    permission_classes = [
        permissions.IsAuthenticated,
        MemberEditOrDjangoModelPermissions,
    ]
    ordering_fields = SearchableModelViewSet.ordering_fields + ("name",)
    ordering: tuple[str, ...] = ("name",)
    search_fields = ["name", "address__line1", "address__country", "address__region"]

    serializers = {
        "retrieve": OrganizationDetailSerializer,
        "list": OrganizationListSerializer,
        "create": OrganizationDetailSerializer,
        "update": OrganizationDetailSerializer,
        "partial_update": OrganizationDetailSerializer,
    }


class AddressViewSet(viewsets.ReadOnlyModelViewSet):
    queryset = Address.objects.all().order_by("id")
    serializer_class = AddressSerializer
    permission_classes = [
        permissions.IsAuthenticated,
        permissions.DjangoModelPermissions,
    ]



from rest_framework.pagination import PageNumberPagination


class PageNumberWithSizePagination(PageNumberPagination):
    page_size = 50
    page_size_query_param = "page_size"
    max_page_size = 200



from .member import Member
from .feedback import Feedback
from .organization import Organization, Address
from .utils import TimesStampMixin, make_zip, content_file_name

__all__ = [
    "Organization",
    "Member",
    "Address",
    "Feedback",
    "TimesStampMixin",
    "make_zip",
    "content_file_name",
]



from functools import partial

import PIL

from django.contrib.auth.models import Group, Permission
from django.contrib.contenttypes.models import ContentType
from django.contrib.auth.models import User
from rest_framework import status
from rest_framework.test import APITestCase

from ichec_django_core.models import Member


def generate_image():
    return PIL.Image.linear_gradient("L")


def setup_default_users_and_groups():

    admins = Group.objects.create(name="admins")
    Group.objects.create(name="regular_users")

    Member.objects.create(username="regular_user")
    admin = Member.objects.create(username="admin_user")

    admin.groups.add(admins)
    admin.save()


def add_group_permissions(group_name: str, model: type, permissions: list):

    group = Group.objects.get(name=group_name)
    content_type = ContentType.objects.get_for_model(model)
    for permission_name in permissions:
        permission = Permission.objects.get(
            codename=permission_name, content_type=content_type
        )
        group.permissions.add(permission)
    group.save()


class AuthAPITestCase(APITestCase):
    def authenticate(self, username: str):
        user = User.objects.get(username=username)
        self.client.force_authenticate(user=user)

    def deauthenticate(self):
        self.client.force_authenticate(user=None)

    def authenticated_op(self, username: str, op):
        self.authenticate(username)
        response = op()
        self.deauthenticate()
        return response

    def do_list(self):
        return self.client.get(self.url, format="json")

    def authenticated_list(self, username: str):
        return self.authenticated_op(username, self.do_list)

    def detail(self, resource_id: int):
        return self.client.get(self.url + f"{resource_id}/", format="json")

    def authenticated_detail(self, username: str, resource_id: int):
        return self.authenticated_op(username, partial(self.detail, resource_id))

    def create(self, data: dict, format="json"):
        return self.client.post(self.url, data, format=format)

    def update(self, resource_id: int, data: dict, format="json"):
        return self.client.put(self.url + f"{resource_id}/", data, format=format)

    def authenticated_update(self, username: str, resource_id: int, data: dict):
        return self.authenticated_op(username, partial(self.update, resource_id, data))

    def put_file(self, resource_id: int, field: str, data, filename):
        return self.client.put(
            f"{self.url}{resource_id}/{field}/upload",
            data,
            headers={"Content-Disposition": f'attachment; filename="{filename}"'},
        )

    def authenticated_create(self, username: str, data: dict, format="json"):
        return self.authenticated_op(username, partial(self.create, data, format))

    def authenticated_put_file(
        self, username: str, resource_id: int, field: str, data, filename: str
    ):
        return self.authenticated_op(
            username, partial(self.put_file, resource_id, field, data, filename)
        )

    def assert_200(self, response):
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        return response.data

    def assert_201(self, response):
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        return response.data

    def assert_204(self, response):
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

    def assert_401(self, response):
        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)

    def assert_403(self, response):
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)



from rest_framework import serializers

from ichec_django_core.models import Feedback

from .core import SERIALIZER_BASE_FIELDS


class FeedbackSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = Feedback
        fields = SERIALIZER_BASE_FIELDS + ("creator", "comments")
        read_only_fields = SERIALIZER_BASE_FIELDS + ("creator",)



from rest_framework import permissions


class DjangoModelPermissionsWithView(permissions.DjangoModelPermissions):
    perms_map = {
        "GET": ["%(app_label)s.view_%(model_name)s"],
        "OPTIONS": [],
        "HEAD": [],
        "POST": ["%(app_label)s.add_%(model_name)s"],
        "PUT": ["%(app_label)s.change_%(model_name)s"],
        "PATCH": ["%(app_label)s.change_%(model_name)s"],
        "DELETE": ["%(app_label)s.delete_%(model_name)s"],
    }


def get_permission(model_t, perm_t: str) -> str:
    return f"{model_t._meta.app_label}.{perm_t}_{model_t._meta.model_name}"


class CustomDjangoModelPermissions(permissions.DjangoModelPermissions):

    intercept_methods: list = []
    safe_methods: list = ["HEAD", "OPTIONS"]

    def has_authenticated_user(self, request) -> bool:
        return request.user and request.user.is_authenticated

    def method_handled_by_object(self, method: str) -> bool:
        return method in self.intercept_methods

    def is_safe_method(self, method: str) -> bool:
        return method in self.safe_methods

    def has_permission(self, request, view):

        if not self.has_authenticated_user(request):
            return False

        # Fall through to allow permission checks to happen at object level
        if self.method_handled_by_object(request.method):
            return True

        return super().has_permission(request, view)

    def has_object_permission(self, request, view, obj):

        # Allow metadata checks on endpoint
        if self.is_safe_method(request.method):
            return True

        if not self.has_authenticated_user(request):
            return super().has_permission(request, view)

        # If the method is handled by the object, then dispatch delegate the check to it
        if self.method_handled_by_object(request.method) and self.object_perm_check(
            obj, request
        ):
            return True

        # Otherwise fall back to Django model permissions
        return super().has_permission(request, view)

    def object_perm_check(self, obj, request) -> bool:
        raise NotImplementedError


class FullOrDjangoModelPermissions(CustomDjangoModelPermissions):

    intercept_methods: list = ["PUT", "PATCH", "DELETE"]


class EditOrDjangoModelPermissions(permissions.DjangoModelPermissions):

    intercept_methods: list = ["PUT", "PATCH"]


class OwnerFullOrDjangoModelPermissions(FullOrDjangoModelPermissions):

    owner_field = "owner"

    def object_perm_check(self, obj, request) -> bool:
        return getattr(obj, self.owner_field).id == request.user.id


class SelfEditOrDjangoModelPermissions(EditOrDjangoModelPermissions):

    def object_perm_check(self, obj, request) -> bool:
        return obj.id == request.user.id


class MemberEditOrDjangoModelPermissions(EditOrDjangoModelPermissions):

    member_field = "members"

    def object_perm_check(self, obj, request) -> bool:
        return request.user.id in [m.id for m in getattr(obj, self.member_field)]



# Generated by Django 5.2.3 on 2025-10-31 16:00

import functools
import ichec_django_core.models.utils
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("ichec_django_core", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="member",
            name="profile",
            field=models.ImageField(
                null=True,
                upload_to=functools.partial(
                    ichec_django_core.models.utils.content_file_name,
                    *("profile",),
                    **{},
                ),
            ),
        ),
    ]



from datetime import datetime

from pydantic import BaseModel


class File:

    def get_content(self):
        pass


class Resource(BaseModel, frozen=True):

    id: int


_TYPE_TO_PATH: dict = {}

_PATH_TO_TYPES: dict = {}


def register_types(label: str, types: dict):

    if "create" not in types:
        types["create"] = types["detail"]
    if "list" not in types:
        types["list"] = types["detail"]

    _PATH_TO_TYPES[label] = types

    for t in types.values():
        _TYPE_TO_PATH[t.__name__] = label


def get_path_from_item(resource: BaseModel) -> str:
    return _TYPE_TO_PATH[type(resource).__name__]


def get_path_from_type(item_t) -> str:
    return _TYPE_TO_PATH[item_t.__name__]


def get_type(path: str, action: str):
    return _PATH_TO_TYPES[path][action]


class Identifiable(Resource, frozen=True):
    url: str


class Timestamped(Identifiable, frozen=True):
    created_at: datetime
    updated_at: datetime


class MemberBase(BaseModel, frozen=True):
    username: str
    email: str
    first_name: str
    last_name: str | None = None
    phone: str | None = None


class MemberCreate(MemberBase, frozen=True):
    profile: str | None = None


class MemberList(Timestamped, MemberBase, frozen=True):
    organizations: list[str] = []
    profile: str | None = None
    thumbnail: str | None = None


class MemberDetail(MemberList, frozen=True):
    permissions: list[str] = []


class GroupBase(BaseModel, frozen=True):
    name: str


class GroupCreate(GroupBase, frozen=True):
    pass


class GroupDetail(Timestamped, GroupBase, frozen=True):
    pass


class AddressBase(BaseModel, frozen=True):
    line1: str
    line2: str | None = None
    line3: str | None = None
    city: str | None = None
    region: str
    postcode: str | None = None
    country: str


class AddressCreate(AddressBase, frozen=True):
    pass


class AddressDetail(Timestamped, AddressBase, frozen=True):
    pass


class OrganizationBase(BaseModel, frozen=True):
    name: str
    acronym: str | None = None
    description: str | None = None
    website: str | None = None
    members: list[str] = []


class OrganizationCreate(OrganizationBase, frozen=True):
    address: AddressCreate


class OrganizationDetail(Timestamped, frozen=True):
    address: AddressDetail


register_types(
    "members", {"create": MemberCreate, "detail": MemberDetail, "list": MemberList}
)
register_types("groups", {"detail": GroupDetail, "create": GroupCreate})
register_types(
    "organizations", {"create": OrganizationCreate, "detail": OrganizationDetail}
)



from django.urls import include, path
from django.contrib import admin
from django.conf import settings

from .view_sets import (
    MemberViewSet,
    MemberProfileDownloadView,
    MemberProfileThumbnailDownloadView,
    MemberProfileUploadView,
    GroupViewSet,
    OrganizationViewSet,
    FeedbackViewSet,
    AddressViewSet,
    CustomAuthToken,
)


def register_drf_views(router):
    router.register(r"groups", GroupViewSet)
    router.register(r"organizations", OrganizationViewSet)
    router.register(r"members", MemberViewSet)
    router.register(r"addresses", AddressViewSet)
    router.register(r"feedback", FeedbackViewSet)
    return router


urlpatterns = [
    path("api-token-auth/", CustomAuthToken.as_view()),
    path(f"{settings.API_AUTH_URL}/", include("rest_framework.urls")),
    path(f"{settings.ADMIN_URL}/", admin.site.urls),
    path(
        r"api/members/<int:pk>/profile",
        MemberProfileDownloadView.as_view(),
        name="member_profiles",
    ),
    path(
        r"api/members/<int:pk>/profile/thumbnail",
        MemberProfileThumbnailDownloadView.as_view(),
        name="member_profile_thumbnails",
    ),
    path(
        r"api/members/<int:pk>/profile/upload",
        MemberProfileUploadView.as_view(),
        name="member_profiles_upload",
    ),
]

if settings.WITH_OIDC:
    urlpatterns += [path("oidc/", include("mozilla_django_oidc.urls"))]



import json

from ichec_django_core.utils.test_utils.test_client import (
    AuthAPITestCase,
    setup_default_users_and_groups,
)


class GroupViewTests(AuthAPITestCase):
    def setUp(self):
        self.url = "/api/groups/"
        setup_default_users_and_groups()

    def test_list_not_authenticated(self):
        self.assert_401(self.do_list())

    def test_detail_not_authenticated(self):
        self.assert_401(self.detail(1))

    def test_list_authenticated(self):
        self.assert_200(self.authenticated_list("regular_user"))

    def test_detail_authenticated(self):
        self.assert_200(self.authenticated_detail("regular_user", 1))

    def test_create_not_authenticated(self):
        data = {"name": "My Group"}
        self.assert_401(self.create(data))

    def test_create_authenticated_no_permission(self):
        data = {"name": "My Group"}
        self.assert_403(self.authenticated_create("regular_user", data))

    def test_filter_user(self):
        self.authenticate("regular_user")
        response = self.client.get(self.url + "?user=1", format="json")
        self.deauthenticate()
        self.assert_200(response)
        content = json.loads(response.content)
        print(content)
        self.assertEqual(content["count"], 1)



import functools

from django.db import models
from django.contrib.auth.models import User, Group
from django.db.models.signals import post_save
from django.dispatch import receiver
from django.conf import settings

from rest_framework.authtoken.models import Token

from .utils import TimesStampMixin, generate_thumbnail, content_file_name


@receiver(post_save, sender=settings.AUTH_USER_MODEL)
def create_auth_token(sender, instance=None, created=False, **kwargs):
    """
    Gives each newly created user an auth token by default
    """

    if created:
        Token.objects.create(user=instance)


class Member(User, TimesStampMixin):
    phone = models.CharField(max_length=100, blank=True, null=True)
    profile = models.ImageField(
        null=True, upload_to=functools.partial(content_file_name, "profile")
    )
    profile_thumbnail = models.ImageField(null=True)

    class Meta:
        verbose_name = "Portal Member"
        verbose_name_plural = "Portal Members"

    def __str__(self):
        return self.username

    @classmethod
    def post_create(cls, sender, instance, created, *args, **kwargs):

        if not created:
            return

        group = Group.objects.filter(name="regular_users").first()
        if group:
            instance.groups.add(group)

    def save(self, *args, **kwargs):
        if not self.profile:
            self.profile_thumbnail = None
        else:
            self.profile_thumbnail.name = generate_thumbnail(
                self, "profile", self.profile
            )
        super().save(*args, **kwargs)


post_save.connect(Member.post_create, sender=Member)



from django.contrib.auth.models import Group

from rest_framework import permissions, viewsets, filters, status
from rest_framework.response import Response

from ichec_django_core.serializers import GroupSerializer

from .permissions import get_permission

BASE_ORDERING_FIELDS = ("id", "created_at", "updated_at")


class BaseModelViewSet(viewsets.ModelViewSet):

    def filter_self_or_permission(self, queryset, model_t):
        if self.request.method == "GET":
            if self.request.user:
                if not self.has_model_view_permission(model_t):
                    return self.filter_self(queryset)
            else:
                return None
        return queryset

    def filter_self(self, queryset):
        return queryset

    def has_model_view_permission(self, model_t) -> bool:
        return self.self_has_permission(get_permission(model_t, "view"))

    def self_has_permission(self, perm: str) -> bool:
        return self.request.user.has_perm(perm)

    def get_serializer_class(self):
        if hasattr(self, "serializers"):
            return self.serializers.get(self.action, self.serializer_class)
        return super().get_serializer_class()

    def create(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)

        self.check_field_permissions(request, serializer)

        self.perform_create(serializer)
        headers = self.get_success_headers(serializer.data)
        return Response(
            serializer.data, status=status.HTTP_201_CREATED, headers=headers
        )

    def update(self, request, *args, **kwargs):
        partial = kwargs.pop("partial", False)
        instance = self.get_object()
        serializer = self.get_serializer(instance, data=request.data, partial=partial)
        serializer.is_valid(raise_exception=True)

        self.check_field_permissions(request, serializer)

        self.perform_update(serializer)

        if getattr(instance, "_prefetched_objects_cache", None):
            # If 'prefetch_related' has been applied to a queryset, we need to
            # forcibly invalidate the prefetch cache on the instance.
            instance._prefetched_objects_cache = {}

        return Response(serializer.data)

    def check_field_permissions(self, _request, _serializer):
        pass


class SearchableModelViewSet(BaseModelViewSet):
    filter_backends = [filters.OrderingFilter, filters.SearchFilter]
    ordering_fields: tuple[str, ...] = BASE_ORDERING_FIELDS
    ordering: tuple[str, ...] = ("id",)


class GroupViewSet(SearchableModelViewSet):
    queryset = Group.objects.all()
    serializer_class = GroupSerializer
    permission_classes = [
        permissions.IsAuthenticated,
        permissions.DjangoModelPermissions,
    ]

    ordering_fields: tuple[str, ...] = ("name",)
    ordering: tuple[str, ...] = ("name",)
    search_fields: tuple[str, ...] = ("name",)

    def get_queryset(self):
        queryset = Group.objects.all()
        user_id = self.request.query_params.get("user")
        if user_id is not None:
            queryset = queryset.filter(user__id=user_id)
        return queryset



from .models import (
    MemberCreate,
    MemberDetail,
    AddressCreate,
    OrganizationCreate,
    GroupCreate,
)


def create_members(count: int = 100, offset: int = 0) -> list[MemberCreate]:

    content = []
    for idx in range(offset, count + offset):
        content.append(
            MemberCreate(
                username=f"member_{idx}",
                email=f"member_{idx}@example.com",
                first_name="Script",
                last_name=f"User {idx}",
            )
        )
    return content


def create_groups(count: int = 100, offset: int = 0) -> list[GroupCreate]:
    content = []

    for idx in range(offset, count + offset):
        content.append(GroupCreate(name=f"Script Group {idx}"))
    return content


def create_organizations(
    count: int = 100, offset: int = 0, possible_members: tuple[MemberDetail, ...] = ()
) -> list[OrganizationCreate]:
    content = []

    for idx in range(offset, count + offset):

        address = AddressCreate(
            line1="Apartment 123",
            line2="123 Street",
            city="City",
            region="Region",
            postcode="abc123",
            country="IE",
        )

        members = []
        if possible_members:
            num_members = len(possible_members) - 1
            ids = set(
                [
                    idx % num_members,
                    (idx + 1) % num_members,
                    (idx + 2) % num_members,
                    (idx + 4) % num_members,
                ]
            )
            members = [possible_members[jdx].url for jdx in ids]

        item = OrganizationCreate(
            name=f"Organization {idx}",
            acronym=f"ORG {idx}",
            description=f"Description of org {idx}",
            address=address,
            website=f"www.org{idx}.com",
            members=members,
        )

        content.append(item)
    return content



from django.test import TestCase

from ichec_django_core.models import Organization, Address


class OrganizationTestCase(TestCase):
    def setUp(self):

        address = Address.objects.create(line1="1234 Street", region="My Region")
        Organization.objects.create(name="my_org", address=address)

    def test_query_model(self):
        org = Organization.objects.get(name="my_org")
        self.assertEqual(org.name, "my_org")
